"""System."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any, cast


@dataclass
class SensorsWindowsSensor:
    """Sensors Windows Sensor."""

    id: str
    name: str
    type: str
    value: float | int | str | None = None


@dataclass
class SensorsWindowsHardware:
    """Sensors Windows Hardware."""

    id: str
    name: str
    type: str
    subhardware: list[SensorsWindowsHardware]
    sensors: list[SensorsWindowsSensor]

    def __post_init__(self):
        """Post Init."""
        if isinstance(self.subhardware, list) and all(
            isinstance(item, dict) for item in self.subhardware
        ):
            new_subhardware: list[SensorsWindowsHardware] = []
            for s in self.subhardware:
                subhardware: dict = cast(dict, s)
                new_subhardware.append(SensorsWindowsHardware(**subhardware))
            self.subhardware = new_subhardware

        if isinstance(self.sensors, list) and all(
            isinstance(item, dict) for item in self.sensors
        ):
            new_sensors: list[SensorsWindowsSensor] = []
            for s in self.sensors:
                sensor: dict = cast(dict, s)
                new_sensors.append(SensorsWindowsSensor(**sensor))
            self.sensors = new_sensors


@dataclass
class SensorsNVIDIAChipset:
    """Sensors NVIDIA Chipset."""

    id: int
    name: str
    flags: str
    vendor_id: int
    vendor_name: str


@dataclass
class SensorsNVIDIADisplay:
    """Sensors NVIDIA Display."""

    id: int
    name: str
    active: bool
    available: bool
    connected: bool
    dynamic: bool
    aspect_horizontal: int
    aspect_vertical: int
    brightness_current: int
    brightness_default: int
    brightness_max: int
    brightness_min: int
    color_depth: str
    connection_type: str
    pixel_clock: int
    refresh_rate: int
    resolution_horizontal: int
    resolution_vertical: int


@dataclass
class SensorsNVIDIADriver:
    """Sensors NVIDIA Driver."""

    branch_version: str
    interface_version: str
    version: int


@dataclass
class SensorsNVIDIAGPU:
    """Sensors NVIDIA GPU."""

    id: int
    name: str
    bios_oem_revision: int | None = None
    bios_revision: int | None = None
    bios_version: str | None = None
    current_fan_speed_level: int | None = None
    current_fan_speed_rpm: int | None = None
    driver_model: int | None = None
    memory_available: int | None = None
    memory_capacity: int | None = None
    memory_maker: str | None = None
    serial: str | None = None
    system_type: str | None = None
    type: str | None = None


@dataclass
class SensorsNVIDIA:
    """Sensors NVIDIA."""

    chipset: SensorsNVIDIAChipset | None = None
    displays: list[SensorsNVIDIADisplay] | None = None
    driver: SensorsNVIDIADriver | None = None
    gpus: list[SensorsNVIDIAGPU] | None = None

    def __post_init__(self):
        """Post Init."""
        if isinstance(self.chipset, dict):
            self.chipset = SensorsNVIDIAChipset(**self.chipset)

        if isinstance(self.displays, list) and all(
            isinstance(item, dict) for item in self.displays
        ):
            new_displays: list[SensorsNVIDIADisplay] = []
            for s in self.displays:
                display: dict = cast(dict, s)
                new_displays.append(SensorsNVIDIADisplay(**display))
            self.displays = new_displays

        if isinstance(self.driver, dict):
            self.driver = SensorsNVIDIADriver(**self.driver)

        if isinstance(self.gpus, list) and all(
            isinstance(item, dict) for item in self.gpus
        ):
            new_gpus: list[SensorsNVIDIAGPU] = []
            for s in self.gpus:
                gpu: dict = cast(dict, s)
                new_gpus.append(SensorsNVIDIAGPU(**gpu))
            self.gpus = new_gpus


@dataclass
class SensorsWindows:
    """Sensors Windows."""

    hardware: list[SensorsWindowsHardware] | None = None
    nvidia: SensorsNVIDIA | None = None

    def __post_init__(self):
        """Post Init."""
        if isinstance(self.hardware, list) and all(
            isinstance(item, dict) for item in self.hardware
        ):
            new_hardware: list[SensorsWindowsHardware] = []
            for s in self.hardware:
                hardware: dict = cast(dict, s)
                new_hardware.append(SensorsWindowsHardware(**hardware))
            self.hardware = new_hardware

        if isinstance(self.nvidia, dict):
            self.nvidia = SensorsNVIDIA(**self.nvidia)


@dataclass
class Sensors:
    """Sensors."""

    # TODO: Add fan model
    fans: Any | None = None
    # TODO: Add temperatures model
    temperatures: Any | None = None
    windows_sensors: SensorsWindows | None = None

    def __post_init__(self):
        """Post Init."""
        if isinstance(self.windows_sensors, dict):
            self.windows_sensors = SensorsWindows(**self.windows_sensors)



"""GPUs."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class GPU:
    """GPU."""

    id: str
    name: str
    core_clock: float | None = None
    core_load: float | None = None
    fan_speed: float | None = None
    memory_clock: float | None = None
    memory_load: float | None = None
    memory_free: float | None = None
    memory_used: float | None = None
    memory_total: float | None = None
    power_usage: float | None = None
    temperature: float | None = None



"""Modules."""

from __future__ import annotations

from dataclasses import dataclass
from enum import StrEnum

from .battery import Battery
from .cpu import CPU
from .disks import Disks
from .displays import Display
from .gpus import GPU
from .media import Media
from .memory import Memory
from .networks import Networks
from .processes import Process
from .sensors import Sensors
from .system import System


class Module(StrEnum):
    """Module Enum."""

    BATTERY = "battery"
    CPU = "cpu"
    DISKS = "disks"
    DISPLAYS = "displays"
    GPUS = "gpus"
    MEDIA = "media"
    MEMORY = "memory"
    NETWORKS = "networks"
    PROCESSES = "processes"
    SENSORS = "sensors"
    SYSTEM = "system"


@dataclass
class GetData:
    """Get Data."""

    modules: list[Module]


@dataclass
class RegisterDataListener(GetData):
    """Register Data Listener."""


@dataclass
class ModulesData:
    """Data."""

    battery: Battery | None = None
    cpu: CPU | None = None
    disks: Disks | None = None
    displays: list[Display] | None = None
    gpus: list[GPU] | None = None
    media: Media | None = None
    memory: Memory | None = None
    networks: Networks | None = None
    processes: list[Process] | None = None
    sensors: Sensors | None = None
    system: System | None = None



"""Fixture for processes module."""
from systembridgemodels.modules.processes import Process

FIXTURE_PROCESSES = [
    Process(
        id=1234,
        name="name",
        cpu_usage=12.3,
        created=12.3,
        memory_usage=12.3,
        path="/path",
        status="running",
        username="username",
        working_directory="/working/directory",
    )
]



"""Battery."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class Battery:
    """Battery."""

    is_charging: bool | None = None
    percentage: float | None = None
    time_remaining: float | None = None



"""Fixture for memory module."""
from systembridgemodels.modules.memory import Memory, MemorySwap, MemoryVirtual

FIXTURE_MEMORY = Memory(
    swap=MemorySwap(
        total=1000,
        used=500,
        free=500,
        percent=50.0,
        sin=100,
        sout=100,
    ),
    virtual=MemoryVirtual(
        total=1000,
        available=500,
        percent=50.0,
        used=500,
        free=500,
        active=500,
        inactive=500,
        buffers=500,
        cached=500,
        wired=500,
        shared=500,
    ),
)



"""Test the action model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.action import Action


def test_action(snapshot: SnapshotAssertion):
    """Test action."""
    action = Action(
        command="command",
        data={"key": "value"},
        label="label",
    )
    assert isinstance(action, Action)
    assert action == snapshot



"""Processes."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class Process:
    """Process."""

    id: float
    name: str | None = None
    cpu_usage: float | None = None
    created: float | None = None
    memory_usage: float | None = None
    path: str | None = None
    status: str | None = None
    username: str | None = None
    working_directory: str | None = None



"""Test the settings model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.settings import Settings, SettingsAPI


def test_settings(snapshot: SnapshotAssertion):
    """Test the settings."""
    settings = Settings(
        api=SettingsAPI(
            token="token",
        ),
    )
    assert isinstance(settings, Settings)
    assert settings == snapshot


def test_settings_token():
    """Test the settings token."""
    settings = Settings()
    assert settings.api.token != ""



"""Media."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class Media:
    """Media Info."""

    album_artist: str | None = None
    album_title: str | None = None
    artist: str | None = None
    duration: float | None = None
    is_fast_forward_enabled: bool | None = None
    is_next_enabled: bool | None = None
    is_pause_enabled: bool | None = None
    is_play_enabled: bool | None = None
    is_previous_enabled: bool | None = None
    is_rewind_enabled: bool | None = None
    is_stop_enabled: bool | None = None
    playback_rate: float | None = None
    position: float | None = None
    repeat: str | None = None
    shuffle: bool | None = None
    status: str | None = None
    subtitle: str | None = None
    thumbnail: str | None = None
    title: str | None = None
    track_number: int | None = None
    type: str | None = None
    updated_at: float | None = None



"""Test the OpenPath model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.open_path import OpenPath


def test_open_path(snapshot: SnapshotAssertion):
    """Test the open_path."""
    open_path = OpenPath(
        path="/path/to/file",
    )
    assert isinstance(open_path, OpenPath)
    assert open_path == snapshot



"""Fixture for networks module."""

from systembridgemodels.modules.networks import (
    Network,
    NetworkAddress,
    NetworkConnection,
    NetworkIO,
    Networks,
    NetworkStats,
)

FIXTURE_NETWORKS = Networks(
    connections=[
        NetworkConnection(
            fd=1,
            family=2,
            type=3,
            laddr="laddr",
            raddr="raddr",
            status="status",
            pid=4,
        )
    ],
    io=NetworkIO(
        bytes_sent=1,
        bytes_recv=2,
        packets_sent=3,
        packets_recv=4,
        errin=5,
        errout=6,
        dropin=7,
        dropout=8,
    ),
    networks=[
        Network(
            name="name",
            addresses=[
                NetworkAddress(
                    address="address",
                    netmask="netmask",
                    broadcast="broadcast",
                    ptp="ptp",
                )
            ],
            stats=NetworkStats(
                isup=True,
                duplex="duplex",
                speed=1,
                mtu=2,
                flags=[
                    "flag1",
                    "flag2",
                ],
            ),
        )
    ],
)



"""
Provides systembridgemodels version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgemodels` to change this file.

from incremental import Version

__version__ = Version("systembridgemodels", 4, 2, 2)
__all__ = ["__version__"]



"""Response."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any


@dataclass
class Response:
    """Response."""

    id: str
    type: str
    data: dict[str, Any] | list[Any]
    subtype: str | None = None
    message: str | None = None
    module: str | None = None



"""Test the media_get_files model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.media_get_files import MediaGetFiles


def test_media_get_files(snapshot: SnapshotAssertion):
    """Test the media_get_files."""
    media_get_files = MediaGetFiles(
        base="music",
        path="path/to/file",
    )
    assert isinstance(media_get_files, MediaGetFiles)
    assert media_get_files == snapshot



"""System."""

from __future__ import annotations

from dataclasses import dataclass
from enum import StrEnum


class RunMode(StrEnum):
    """Run Mode."""

    STANDALONE = "standalone"
    PYTHON = "python"


@dataclass
class SystemUser:
    """System User."""

    name: str
    active: bool
    terminal: str
    host: str
    started: float
    pid: float


@dataclass
class System:
    """System."""

    boot_time: float
    fqdn: str
    hostname: str
    ip_address_4: str
    mac_address: str
    platform_version: str
    platform: str
    uptime: float
    users: list[SystemUser]
    uuid: str
    version: str
    camera_usage: list[str] | None = None
    ip_address_6: str | None = None
    pending_reboot: bool | None = None
    run_mode: RunMode | None = None
    version_latest_url: str | None = None
    version_latest: str | None = None
    version_newer_available: bool | None = None



"""Test the OpenPath model."""

from dataclasses import asdict

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.fixtures.notification import FIXTURE_NOTIFICATION
from systembridgemodels.notification import Notification


def test_notification(snapshot: SnapshotAssertion):
    """Test the notification."""
    notification = FIXTURE_NOTIFICATION
    assert isinstance(notification, Notification)
    assert notification == snapshot


def test_notification_dict(snapshot: SnapshotAssertion):
    """Test notification dict."""
    notification_dict = asdict(FIXTURE_NOTIFICATION)
    assert isinstance(notification_dict, dict)
    assert notification_dict == snapshot

    notification_converted = Notification(**notification_dict)
    assert isinstance(notification_converted, Notification)
    assert notification_converted == snapshot(
        name="notification-dict-converted",
    )



"""Test fixtures for the disks module."""
from systembridgemodels.modules.disks import (
    Disk,
    DiskIOCounters,
    DiskPartition,
    Disks,
    DiskUsage,
)

FIXTURE_DISKS = Disks(
    devices=[
        Disk(
            name="name",
            partitions=[
                DiskPartition(
                    device="device",
                    mount_point="mountpoint",
                    filesystem_type="fstype",
                    options="options",
                    max_file_size=1,
                    max_path_length=2,
                    usage=DiskUsage(
                        total=1,
                        used=2,
                        free=3,
                        percent=40.2,
                    ),
                ),
            ],
            io_counters=DiskIOCounters(
                read_bytes=1,
                write_bytes=2,
                read_count=3,
                write_count=4,
                read_time=5,
                write_time=6,
            ),
        )
    ],
    io_counters=DiskIOCounters(
        read_bytes=1,
        write_bytes=2,
        read_count=3,
        write_count=4,
        read_time=5,
        write_time=6,
    ),
)



"""Fixture for notification."""
from systembridgemodels.notification import Action, Audio, Notification

FIXTURE_NOTIFICATION = Notification(
    title="Title",
    message="Message",
    icon="https://www.example.com/icon.png",
    image="https://www.example.com/image.png",
    actions=[Action(command="COMMAND_TEST", label="LABEL_TEST", data={"test": "test"})],
    timeout=1000,
    audio=Audio(
        source="https://www.example.com/audio.mp3",
        volume=100,
    ),
)



"""Disks."""
from __future__ import annotations

from dataclasses import dataclass
from typing import cast


@dataclass
class DiskIOCounters:
    """Disk IO Counters."""

    read_count: int
    write_count: int
    read_bytes: int
    write_bytes: int
    read_time: int
    write_time: int


@dataclass
class DiskUsage:
    """Disk Usage."""

    total: int
    used: int
    free: int
    percent: float


@dataclass
class DiskPartition:
    """Disk Partition."""

    device: str
    mount_point: str
    filesystem_type: str
    options: str
    max_file_size: int
    max_path_length: int
    usage: DiskUsage | None = None

    def __post_init__(self) -> None:
        """Post Init."""
        if isinstance(self.usage, dict):
            self.usage = DiskUsage(**self.usage)


@dataclass
class Disk:
    """Disk."""

    name: str
    partitions: list[DiskPartition]
    io_counters: DiskIOCounters | None = None

    def __post_init__(self) -> None:
        """Post Init."""
        if isinstance(self.partitions, list) and all(
            isinstance(item, dict) for item in self.partitions
        ):
            new_partitions: list[DiskPartition] = []
            for p in self.partitions:
                partition: dict = cast(dict, p)
                new_partitions.append(DiskPartition(**partition))
            self.partitions = new_partitions

        if isinstance(self.io_counters, dict):
            self.io_counters = DiskIOCounters(**self.io_counters)


@dataclass
class Disks:
    """Disks."""

    devices: list[Disk]
    io_counters: DiskIOCounters | None = None

    def __post_init__(self) -> None:
        """Post Init."""
        if isinstance(self.devices, list) and all(
            isinstance(item, dict) for item in self.devices
        ):
            new_devices: list[Disk] = []
            for d in self.devices:
                device: dict = cast(dict, d)
                partitions: list[DiskPartition] = []
                for partition in device.get("partitions", []):
                    partitions.append(DiskPartition(**partition))
                io_counters = device.get("io_counters")
                new_devices.append(
                    Disk(
                        name=device["name"],
                        partitions=partitions,
                        io_counters=(
                            DiskIOCounters(**io_counters) if io_counters else None
                        ),
                    )
                )
            self.devices = new_devices

        if isinstance(self.io_counters, dict):
            self.io_counters = DiskIOCounters(**self.io_counters)



"""Test the media_play model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.media_play import MediaPlay


def test_media_play(snapshot: SnapshotAssertion):
    """Test the media_play."""
    media_play = MediaPlay(
        url="https://www.example.com/audio.mp3",
        album="Album",
        artist="Artist",
        autoplay=True,
        cover="https://www.example.com/cover.png",
        title="Title",
        volume=100,
    )
    assert isinstance(media_play, MediaPlay)
    assert media_play == snapshot



"""Test the media_get_file model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels.media_get_file import MediaGetFile


def test_media_get_file(snapshot: SnapshotAssertion):
    """Test the media_get_file."""
    media_get_file = MediaGetFile(
        base="music",
        path="path/to/file",
    )
    assert isinstance(media_get_file, MediaGetFile)
    assert media_get_file == snapshot



"""Test the update model."""

from syrupy.assertion import SnapshotAssertion

from systembridgemodels._version import __version__
from systembridgemodels.update import Update


def test_update(snapshot: SnapshotAssertion):
    """Test the update."""
    update = Update(
        version=__version__.public(),
    )
    assert isinstance(update, Update)
    assert update == snapshot



"""Fixture for system module."""

from systembridgemodels.modules.system import RunMode, System, SystemUser

FIXTURE_SYSTEM = System(
    boot_time=1234,
    fqdn="hostname.local",
    hostname="hostname",
    ip_address_4="192.168.1.100",
    mac_address="00:00:00:00:00:00",
    platform_version="1.0.0",
    platform="platform",
    run_mode=RunMode.STANDALONE,
    uptime=1234,
    users=[
        SystemUser(
            name="username",
            active=True,
            terminal="terminal",
            host="host",
            started=1234,
            pid=1234,
        )
    ],
    uuid="uuid",
    version="1.0.0",
    camera_usage=[
        "camera1",
        "camera2",
    ],
    ip_address_6="::1",
    pending_reboot=True,
    version_latest_url="https://github.com/timmo001/system-bridge/releases/latest",
    version_latest="4.99.0",
    version_newer_available=True,
)
